import pyfastchem
import numpy as np
from astropy import constants as const
import pandas as pd


#Saves the FastChem output in an ASCII file
def saveChemistryOutput(file_path,                     #the path to the output file
                        temperature, pressure,         #arrays of temperature and pressure
                        total_element_density,         #array of total element density 
                        mean_molecular_weight,         #array of mean molecular weights 
                        number_densities,              #2D array of number densities
                        fastchem,                      #the FastChem object
                        output_species=None,           #optional array with symbols of species that should be saved
                        additional_columns=None,       #optional, additional columns for the output file
                        additional_columns_desc=None): #the header descriptions of the additional columns
  

  #find out how many additional columns and descriptions we have
  nb_add_columns = 0

  if additional_columns is not None:
    add_columns = np.atleast_1d(additional_columns)

    if add_columns.size > add_columns.shape[0]:
      nb_add_columns = add_columns.shape[0]
    else:
      nb_add_columns = 1
  

  add_columns_desc = np.atleast_1d(additional_columns_desc)


  if (nb_add_columns > 0 and add_columns_desc.size != nb_add_columns) or (additional_columns_desc is None and nb_add_columns > 0):
    print('Warning from saveChemistryOutput: The number of additional column descriptions does not match the number of data columns.')


  #total gas particle number density from the ideal gas law 
  #used to convert the number densities to mixing ratios
  gas_number_density = pressure*1e6 / (const.k_B.cgs * temperature)


  #calculate the mixing ratios from the number densities
  mixing_ratios = np.array(number_densities) / gas_number_density.value[:, None]


  with open(file_path, 'w') as file:
    #file header
    file.write('{0:<16}\t{1:<16}\t{2:<16}\t{3:<16}\t{4:<16}'.format('#P (bar)', 'T (K)', 'n_<tot> (cm-3)', 'n_g (cm-3)', 'm (g/mol)'))

    #print the description of the additional columns
    #use 'unk' if the their number do not correspond to the number of additonal columns
    for i in range (nb_add_columns):
      if add_columns_desc.size == nb_add_columns:
        file.write('\t{0:<16}'.format(additional_columns_desc[i]))
      else:
        file.write('\t{0:<16}'.format('unk'))


    if output_species is None: #save all species
      for j in range(fastchem.getSpeciesNumber()):
        file.write('\t{0:<16}'.format(fastchem.getSpeciesSymbol(j)))
    
      file.write('\n')
    
      #and the chemistry output
      for i in range(pressure.size):
        file.write('{0:1.10e}\t{1:1.10e}\t{2:1.10e}\t{3.value:1.10e}\t{4:1.10e}'.format(pressure[i], 
                                                                                        temperature[i], 
                                                                                        total_element_density[i], 
                                                                                        gas_number_density[i], 
                                                                                        mean_molecular_weight[i]))


        #print the additional columns
        for j in range(nb_add_columns):
          if nb_add_columns == 1:
            file.write('\t{0:1.10e}'.format(additional_columns[i]))
          else:
            file.write('\t{0:1.10e}'.format(additional_columns[j, i]))

      
        for j in range(fastchem.getSpeciesNumber()):
          file.write('\t{0:1.10e}'.format(mixing_ratios[i, j]))
      
        file.write('\n')
   
    else:  #save only selected species
      for species in output_species:
        if fastchem.getSpeciesIndex(species) != pyfastchem.FASTCHEM_UNKNOWN_SPECIES:
          file.write('\t{0:<16}'.format(species))
        else:
          print('Species ', species, ' not found during saving of the chemistry output!')
    
      file.write('\n')
    
      #and the chemistry output
      for i in range(pressure.size):
        file.write('{0:1.10e}\t{1:1.10e}\t{2:1.10e}\t{3.value:1.10e}\t{4:1.10e}'.format(pressure[i], 
                                                                                        temperature[i], 
                                                                                        total_element_density[i], 
                                                                                        gas_number_density[i], 
                                                                                        mean_molecular_weight[i]))


        #print the additional columns
        for j in range(nb_add_columns):
          if nb_add_columns == 1:
            file.write('\t{0:1.10e}'.format(additional_columns[i]))
          else:
            file.write('\t{0:1.10e}'.format(additional_columns[j, i]))


        for species in output_species:
          species_index = fastchem.getSpeciesIndex(species)
          if species_index != pyfastchem.FASTCHEM_UNKNOWN_SPECIES:
            file.write('\t{0:1.10e}'.format(mixing_ratios[i, species_index]))
      
        file.write('\n')


  return None




#Saves the FastChem monitor output in an ASCII file
def saveMonitorOutput(file_path,                     #the path to the output file
                      temperature, pressure,         #arrays of temperature and pressure
                      element_conserved,             #array of int for the element conservation
                      fastchem_flags,                #array of int for the output flags
                      nb_chemistry_iterations,       #array of int for the number of iterations
                      total_element_density,         #array with total element density
                      mean_molecular_weight,         #array with the mean molecular weight
                      fastchem,                      #the FastChem object
                      additional_columns=None,       #optional, additional columns for the output file
                      additional_columns_desc=None): #the header descriptions of the additional columns


  #find out how many additional columns and descriptions we have
  nb_add_columns = 0

  if additional_columns is not None:
    add_columns = np.atleast_1d(additional_columns)

    if add_columns.size > add_columns.shape[0]:
      nb_add_columns = add_columns.shape[0]
    else:
      nb_add_columns = 1


  add_columns_desc = np.atleast_1d(additional_columns_desc)


  if (nb_add_columns > 0 and add_columns_desc.size != nb_add_columns) or (additional_columns_desc is None and nb_add_columns > 0):
    print('Warning from saveMonitorOutput: The number of additional column descriptions does not match the number of data columns.')


  #total gas particle number density from the ideal gas law 
  gas_number_density = pressure*1e6 / (const.k_B.cgs * temperature)


  #convert the 2D element conservation array into a NumPy array
  element_conserved = np.array(element_conserved)


  with open(file_path, 'w') as file:
    #file header
    file.write('{0:<16}{1:<16}{2:<24}{3:<24}{4:<24}{5:<24}{6:<24}{7:<24}{8:<24}'.format('#grid point', 
                                                                                        'c_iterations', 
                                                                                        'c_convergence', 
                                                                                        'elem_conserved', 
                                                                                        'P (bar)', 
                                                                                        'T (K)', 
                                                                                        'n_<tot> (cm-3)', 
                                                                                        'n_g (cm-3)', 
                                                                                        'm (g/mol)'))

    #print the header description of the additional columns
    #use 'unk' if the their number do not correspond to the number of additonal columns
    for i in range (nb_add_columns):
      if add_columns_desc.size == nb_add_columns:
        file.write('{0:<24}'.format(additional_columns_desc[i]))
      else:
        file.write('{0:<24}'.format('unk'))

    #header with element symbols
    for j in range(fastchem.getElementNumber()):
      file.write('{0:<8}'.format(fastchem.getSpeciesSymbol(j)))
    
    file.write('\n')


    output_flags = ['fail', 'ok']
    
    #and the debug output
    for i in range(len(fastchem_flags)):

      if fastchem_flags[i] == pyfastchem.FASTCHEM_SUCCESS:
        c_conv = output_flags[1]
      else:
        c_conv = output_flags[0]

      all_elements_conserved = 1

      if np.isin(0, element_conserved[i], assume_unique=True) : all_elements_conserved = 0

      file.write('{0:<16d}{1:<16d}{2:<24s}{3:<24s}{4:<24.10e}{5:<24.10e}{6:<24.10e}{7.value:<24.10e}{8:<24.10e}'.format(i, 
                                                                                                                  nb_chemistry_iterations[i], 
                                                                                                                  c_conv, 
                                                                                                                  output_flags[all_elements_conserved], 
                                                                                                                  pressure[i], temperature[i], 
                                                                                                                  total_element_density[i], 
                                                                                                                  gas_number_density[i], 
                                                                                                                  mean_molecular_weight[i]))

      #print the additional columns
      for j in range(nb_add_columns):
        if nb_add_columns == 1:
          file.write('{0:<24.10e}'.format(additional_columns[i]))
        else:
          file.write('{0:<24.10e}'.format(additional_columns[j, i]))

      #print element conservation info
      for j in range(fastchem.getElementNumber()):
        file.write('{0:<8s}'.format(output_flags[element_conserved[i,j]]))
      
      file.write('\n')


  return None



#Saves the FastChem output in a pickle file using pandas
def saveChemistryOutputPandas(file_path,                     #the path to the output file
                              temperature, pressure,         #arrays of temperature and pressure
                              total_element_density,         #array of total element density 
                              mean_molecular_weight,         #array of mean molecular weights 
                              number_densities,              #2D array of number densities
                              fastchem,                      #the FastChem object
                              output_species=None,           #optional array with symbols of species that should be saved
                              additional_columns=None,       #optional, additional columns for the output file
                              additional_columns_desc=None): #the header descriptions of the additional columns
  
  # total gas particle number density from the ideal gas law
  # used to convert the number densities to mixing ratios
  gas_number_density = pressure * 1e6 / (const.k_B.cgs * temperature)

  # calculate the mixing ratios from the number densities
  mixing_ratios = np.array(number_densities) / gas_number_density.value[:, None]


  #find out how many additional columns and descriptions we have
  nb_add_columns = 0

  if additional_columns is not None:
    additional_columns = np.atleast_1d(additional_columns)

    if additional_columns.size > additional_columns.shape[0]:
      nb_add_columns = additional_columns.shape[0]
    else:
      nb_add_columns = 1

  add_columns_desc = np.atleast_1d(additional_columns_desc)


  if (add_columns_desc.size != nb_add_columns and nb_add_columns > 0) or (additional_columns_desc is None and nb_add_columns > 0):
    print('Warning from saveChemistryOutputPandas: The number of additional column descriptions does not match the number of data columns.')


  #general column headers
  columns = ['P (bar)', 'T (K)', 'n_<tot> (cm-3)', 'n_g (cm-3)', 'm (g/mol)']


  #add the descriptions of the additional columns to the header
  #if their number does not equal the number of additional columns, add 'unk'
  for i in range(nb_add_columns):
    if add_columns_desc.size == nb_add_columns and additional_columns_desc is not None:
      columns.append(add_columns_desc[(i)])
    else:
      columns.append('unk')


  #header for species symbols
  if output_species is None:
    for j in range(fastchem.getSpeciesNumber()):
      columns.append(fastchem.getSpeciesSymbol(j))
  else:
    select_species_id = []

    for species in output_species:
      if fastchem.getSpeciesIndex(species) != pyfastchem.FASTCHEM_UNKNOWN_SPECIES:
        columns.append(species)
        select_species_id.append(fastchem.getSpeciesIndex(species))
      else:
        print('Species ', species, ' not found during saving of the chemistry output!')


  #combine all general data columns
  if additional_columns is None:
    rows = np.vstack([pressure,
                      temperature,
                      total_element_density,
                      gas_number_density.value,
                      mean_molecular_weight]
                    ).T
  else:
    rows = np.vstack([pressure,
                      temperature,
                      total_element_density,
                      gas_number_density.value,
                      mean_molecular_weight,
                      additional_columns]
                    ).T


  #and now we add the mixing ratios
  if output_species is None:
    data = np.hstack([rows, mixing_ratios])
  else:
    data = np.hstack([rows, mixing_ratios[:,select_species_id]])


  #combine column headers and data into a pandas DataFrame
  df = pd.DataFrame(data=data, columns=columns)


  #and finally save it as a pickle file
  df.to_pickle(file_path)

  return None



#Saves the FastChem monitor output in a pickle file using pandas
def saveMonitorOutputPandas(file_path,                     #the path to the output file
                            temperature, pressure,         #arrays of temperature and pressure
                            element_conserved,             #array of int for the element conservation
                            fastchem_flags,                #array of int for the output flags
                            nb_chemistry_iterations,       #array of int for the number of iterations
                            total_element_density,         #array with total element density
                            mean_molecular_weight,         #array with the mean molecular weight
                            fastchem,                      #the FastChem object
                            additional_columns=None,       #optional, additional columns for the output file
                            additional_columns_desc=None): #the header descriptions of the additional columns
  
  # total gas particle number density from the ideal gas law
  # used to convert the number densities to mixing ratios
  gas_number_density = pressure * 1e6 / (const.k_B.cgs * temperature)

  #convert the 2D element conservation array into a NumPy array
  element_conserved = np.array(element_conserved)


  #find out how many additional columns and descriptions we have
  nb_add_columns = 0

  if additional_columns is not None:
    additional_columns = np.atleast_1d(additional_columns)

    if additional_columns.size > additional_columns.shape[0]:
      nb_add_columns = additional_columns.shape[0]
    else:
      nb_add_columns = 1

  add_columns_desc = np.atleast_1d(additional_columns_desc)


  if (add_columns_desc.size != nb_add_columns and nb_add_columns > 0) or (additional_columns_desc is None and nb_add_columns > 0):
    print('Warning from saveMonitorOutputPandas: The number of additional column descriptions does not match the number of data columns.')


  #general column headers
  columns = ['c_iterations', 'c_convergence', 'elem_conserved', 'P (bar)', 'T (K)', 'n_<tot> (cm-3)', 'n_g (cm-3)', 'm (g/mol)']


  #add the descriptions of the additional columns to the header
  #if their number does not equal the number of additional columns, add 'unk'
  for i in range(nb_add_columns):
    if add_columns_desc.size == nb_add_columns and additional_columns_desc is not None:
      columns.append(add_columns_desc[(i)])
    else:
      columns.append('unk')


  nb_elements = fastchem.getElementNumber()

  #header for element symbols
  for j in range(nb_elements):
    columns.append(fastchem.getSpeciesSymbol(j))


  #and the debug output
  all_elements_conserved = np.ones(len(fastchem_flags), dtype=int)

  for i in range(len(fastchem_flags)):
    if np.isin(0, element_conserved[i], assume_unique=True) : all_elements_conserved[i] = 0

  #combine all general data columns
  if additional_columns is None:
    rows = np.vstack([nb_chemistry_iterations,
                      fastchem_flags,
                      all_elements_conserved,
                      pressure,
                      temperature,
                      total_element_density,
                      gas_number_density.value,
                      mean_molecular_weight]
                    ).T
  else:
    rows = np.vstack([nb_chemistry_iterations,
                      fastchem_flags,
                      all_elements_conserved,
                      pressure,
                      temperature,
                      total_element_density,
                      gas_number_density.value,
                      mean_molecular_weight,
                      additional_columns]
                    ).T

  #and now we add the element conservation info
  data = np.hstack([rows, element_conserved])


  #combine column headers and data into a pandas DataFrame
  df = pd.DataFrame(data=data, columns=columns)

  #change some of the datatypes back to int
  df = df.astype({'c_iterations':'int64'})
  df = df.astype({'c_convergence':'int64'})
  df = df.astype({'elem_conserved':'int64'})

  for j in range(nb_elements):
    df = df.astype({fastchem.getSpeciesSymbol(j) :'int64'})

  #and finally save it as a pickle file
  df.to_pickle(file_path)

  return None






 
import pyfastchem
import numpy as np
import os
from save_output import saveChemistryOutput, saveMonitorOutput, saveChemistryOutputPandas, saveMonitorOutputPandas
import matplotlib.pyplot as plt
from astropy import constants as const

#input values for temperature (in K) and pressure (in bar)
#we only use a single values here
temperature_single = 1000
pressure_single = 1

#the range of solar metallicities we want to calculate the chemistry for
metallicity = np.logspace(-1, 2, 100)


#define the directory for the output
#here, we currently use the standard one from FastChem
output_dir = '../output'


#the chemical species we want to plot later
#note that the standard FastChem input files use the Hill notation
plot_species = ['H2O1', 'C1O2', 'C1O1', 'C1H4', 'H3N1']
#for the plot lables, we therefore use separate strings in the usual notation
plot_species_lables = ['H2O', 'CO2', 'CO', 'CH4', 'NH3']


#create a FastChem object
#it needs the locations of the element abundance and equilibrium constants files
#these locations have to be relative to the one this Python script is called from
fastchem = pyfastchem.FastChem('../input/element_abundances_solar.dat', '../input/logK.dat', 1)


#we could also create a FastChem object by using the parameter file
#note, however, that the file locations in the parameter file are relative
#to the location from where this Python script is called from
#fastchem = pyfastchem.FastChem('../input/parameters.dat', 1)


#make a copy of the solar abundances from FastChem
solar_abundances = np.array(fastchem.getElementAbundances())


#allocate the data for the output
nb_points = metallicity.size

number_densities = np.zeros((nb_points, fastchem.getSpeciesNumber()))
total_element_density = np.zeros(nb_points)
mean_molecular_weight = np.zeros(nb_points)
element_conserved = np.zeros((nb_points, fastchem.getElementNumber()), dtype=int)
fastchem_flags = np.zeros(nb_points, dtype=int)
nb_chemistry_iterations = np.zeros(nb_points, dtype=int)

temperature = np.zeros(nb_points)
pressure = np.zeros(nb_points)


#loop over the metallicity values
#since we constantly change the element abundances, we have to call FastChem separately each time
for i in range(0, nb_points):
  element_abundances = np.copy(solar_abundances)
  
  #scale the element abundances, except those of H and He
  for j in range(0, fastchem.getElementNumber()):
    if fastchem.getSpeciesSymbol(j) != 'H' and fastchem.getSpeciesSymbol(j) != 'He':
      element_abundances[j] *= metallicity[i]

  fastchem.setElementAbundances(element_abundances)
  
  #create the input and output structures for FastChem
  input_data = pyfastchem.FastChemInput()
  output_data = pyfastchem.FastChemOutput()

  input_data.temperature = [temperature_single]
  input_data.pressure = [pressure_single]

  fastchem_flag = fastchem.calcDensities(input_data, output_data)
  print("FastChem reports:", pyfastchem.FASTCHEM_MSG[fastchem_flag])


  #copy the FastChem input and output into the pre-allocated arrays
  temperature[i] = input_data.temperature[0]
  pressure[i] = input_data.pressure[0]


  number_densities[i,:] = np.array(output_data.number_densities[0])

  total_element_density[i] = output_data.total_element_density[0]
  mean_molecular_weight[i] = output_data.mean_molecular_weight[0]
  element_conserved[i,:] = output_data.element_conserved[0]
  fastchem_flags[i] = output_data.fastchem_flag[0]
  nb_chemistry_iterations[i] = output_data.nb_chemistry_iterations[0]



#total gas particle number density from the ideal gas law 
gas_number_density = pressure*1e6 / (const.k_B.cgs * temperature)


#check if output directory exists
#create it if it doesn't
os.makedirs(output_dir, exist_ok=True)


#save the monitor output to a file
#we add an additional output column for the metallicity
saveMonitorOutput(output_dir + '/monitor.dat', 
                  temperature, pressure, 
                  element_conserved,
                  fastchem_flags,
                  nb_chemistry_iterations,
                  total_element_density,
                  mean_molecular_weight,
                  fastchem,
                  metallicity, 'M/H')


#this saves the output of all species
#we add an additional column for the metallicity
saveChemistryOutput(output_dir + '/chemistry.dat', 
                    temperature, pressure,
                    total_element_density, 
                    mean_molecular_weight, 
                    number_densities,
                    fastchem, 
                    None, 
                    metallicity, 'M/H')


#save the monitor output to a file
#here, the data is saved as a pandas DataFrame inside a pickle file
#we add an additional output column for the metallicity
# saveMonitorOutputPandas(output_dir + '/monitor.pkl', 
#                   temperature, pressure, 
#                   element_conserved,
#                   fastchem_flags,
#                   nb_chemistry_iterations,
#                   total_element_density,
#                   mean_molecular_weight,
#                   fastchem,
#                   metallicity, 'M/H')


#this would save the output of all species
#here, the data is saved as a pandas DataFrame inside a pickle file
# saveChemistryOutputPandas(output_dir + '/chemistry.pkl', 
#                     temperature, pressure,
#                     total_element_density, 
#                     mean_molecular_weight, 
#                     number_densities,
#                     fastchem, 
#                     None, 
#                     metallicity, 'M/H')


#check the species we want to plot and get their indices from FastChem
plot_species_indices = []
plot_species_symbols = []

for i, species in enumerate(plot_species):
  index = fastchem.getSpeciesIndex(species)

  if index != pyfastchem.FASTCHEM_UNKNOWN_SPECIES:
    plot_species_indices.append(index)
    plot_species_symbols.append(plot_species_lables[i])
  else:
    print("Species", species, "to plot not found in FastChem")


#and plot...
for i in range(0, len(plot_species_symbols)):
  plt.plot(metallicity, number_densities[:, plot_species_indices[i]]/gas_number_density)

plt.xscale('log')
plt.yscale('log')

plt.ylabel("Mixing ratios")
plt.xlabel("M/H")
plt.legend(plot_species_symbols)

plt.show()

#we could also save the figure as a pdf
#plt.savefig(output_dir + '/fastchem_metallicity_fig.pdf')


from glob import glob
import os
import sys
from setuptools import setup
import tempfile
from pybind11.setup_helpers import Pybind11Extension, build_ext
from distutils.dir_util import mkpath
from distutils.errors import CCompilerError
from distutils import sysconfig


__version__ = "2.1"


#Custom build class that provides additional checks for OpenMP
class custom_build_ext(build_ext):
  
  #Compile a test program to determine if C++ compiler supports OpenMP
  def check_openmp_support(self):
    mkpath(self.build_temp)
        
    with tempfile.NamedTemporaryFile(mode='w',
                                     dir=self.build_temp,
                                     prefix='openmptest',
                                     suffix='.cpp') as srcfile:
            
      print("Checking if compiler supports OpenMP")
            
      srcfile.write("""
                    #include <omp.h>
                   
                    int omp_test() 
                    {
                      #pragma omp parallel for
                      for (int i=0; i<5; ++i);
                    
                      return omp_get_num_threads();
                    }
                    """)
     
      srcfile.flush()
            
      try:
        objects = self.compiler.compile([srcfile.name],
                                        extra_postargs=["-fopenmp"],
                                        output_dir="/")
      except CCompilerError:
        print("Compiler does not support OpenMP")
        use_openmp = False
      else:
        print("Enabling OpenMP support")
        use_openmp = True
          
        for o in objects:
          os.remove(o)
    
      return use_openmp

  #Add OpenMP compiler and linker flags if necessary
  def build_extensions(self):
    use_openmp = self.check_openmp_support()
        
    if use_openmp:
      for ext in self.extensions:
        if not ext.extra_compile_args:
          ext.extra_compile_args = []
        ext.extra_compile_args.append('-fopenmp')
        
        if not ext.extra_link_args:
          ext.extra_link_args = []
        ext.extra_link_args.append('-fopenmp')
    
    #Call the build function from the parent class
    build_ext.build_extensions(self)



ext_modules = [
  Pybind11Extension(
    "pyfastchem",
    sorted(glob("fastchem_src/*.cpp") +
           glob("python/fastchem_python_wrapper.cpp")),
    define_macros = [('_SETUP_PY', '1')],
    cxx_std = 11,
    language ='c++',
  ),
]


setup(
  name        = "pyfastchem",
  description = "FastChem, an ultra-fast equilibrium chemistry",
  author      = "Daniel Kitzmann, Joachim Stock, Brett Morris",
  license     = "GPL 3.0",
  url         = "https://github.com/exoclime/FastChem",
  version     = __version__,
  ext_modules = ext_modules,
  cmdclass    = {"build_ext": custom_build_ext}
)




import pyfastchem
from save_output import saveChemistryOutput, saveMonitorOutput, saveChemistryOutputPandas, saveMonitorOutputPandas
import numpy as np
import os
import matplotlib.pyplot as plt
from astropy import constants as const


#some input values for temperature (in K) and pressure (in bar)
temperature = np.full(1000, 500)
pressure = np.logspace(-6, 1, num=1000)


#define the directory for the output
#here, we currently use the standard one from FastChem
output_dir = '../output'


#the chemical species we want to plot later
#note that the standard FastChem input files use the Hill notation
plot_species = ['H2O1', 'C1O2', 'C1O1', 'C1H4', 'H3N1']
#for the plot lables, we therefore use separate strings in the usual notation
plot_species_lables = ['H2O', 'CO2', 'CO', 'CH4', 'NH3']


#create a FastChem object
fastchem = pyfastchem.FastChem('../input/element_abundances_solar.dat', '../input/logK.dat', 1)


#we could also create a FastChem object by using the parameter file
#note, however, that the file locations in the parameter file are relative
#to the location from where this Python script is called from
#fastchem = pyfastchem.FastChem('../input/parameters.dat', 1)



#create the input and output structures for FastChem
input_data = pyfastchem.FastChemInput()
output_data = pyfastchem.FastChemOutput()

input_data.temperature = temperature
input_data.pressure = pressure


#run FastChem on the entire p-T structure
fastchem_flag = fastchem.calcDensities(input_data, output_data)
print("FastChem reports:", pyfastchem.FASTCHEM_MSG[fastchem_flag])


#convert the output into a numpy array
number_densities = np.array(output_data.number_densities)


#total gas particle number density from the ideal gas law 
#used later to convert the number densities to mixing ratios
gas_number_density = pressure*1e6 / (const.k_B.cgs * temperature)


#check if output directory exists
#create it if it doesn't
os.makedirs(output_dir, exist_ok=True)


#save the monitor output to a file
saveMonitorOutput(output_dir + '/monitor.dat', 
                  temperature, pressure, 
                  output_data.element_conserved,
                  output_data.fastchem_flag,
                  output_data.nb_chemistry_iterations,
                  output_data.total_element_density,
                  output_data.mean_molecular_weight,
                  fastchem)

#this would save the output of all species
saveChemistryOutput(output_dir + '/chemistry.dat', 
                    temperature, pressure, 
                    output_data.total_element_density, 
                    output_data.mean_molecular_weight, 
                    output_data.number_densities, 
                    fastchem)

#this saves only selected species (here the species we also plot)
saveChemistryOutput(output_dir + '/chemistry_select.dat', 
                    temperature, pressure, 
                    output_data.total_element_density, 
                    output_data.mean_molecular_weight, 
                    output_data.number_densities, 
                    fastchem,
                    plot_species)


#save the monitor output to a file
#here, the data is saved as a pandas DataFrame inside a pickle file
# saveMonitorOutputPandas(output_dir + '/monitor.pkl', 
#                   temperature, pressure, 
#                   output_data.element_conserved,
#                   output_data.fastchem_flag,
#                   output_data.nb_chemistry_iterations,
#                   output_data.total_element_density,
#                   output_data.mean_molecular_weight,
#                   fastchem)

# #this would save the output of all species
# #here, the data is saved as a pandas DataFrame inside a pickle file
# saveChemistryOutputPandas(output_dir + '/chemistry.pkl',
#                     temperature, pressure,
#                     output_data.total_element_density,
#                     output_data.mean_molecular_weight,
#                     output_data.number_densities,
#                     fastchem)



#check the species we want to plot and get their indices from FastChem
plot_species_indices = []
plot_species_symbols = []

for i, species in enumerate(plot_species):
  index = fastchem.getSpeciesIndex(species)

  if index != pyfastchem.FASTCHEM_UNKNOWN_SPECIES:
    plot_species_indices.append(index)
    plot_species_symbols.append(plot_species_lables[i])
  else:
    print("Species", species, "to plot not found in FastChem")


#and plot...
for i in range(0, len(plot_species_symbols)):
  fig = plt.plot(number_densities[:, plot_species_indices[i]]/gas_number_density, pressure)

plt.xscale('log')
plt.yscale('log')
plt.gca().set_ylim(plt.gca().get_ylim()[::-1])

plt.xlabel("Mixing ratios")
plt.ylabel("Pressure (bar)")
plt.legend(plot_species_symbols)

plt.show()

#we could also save the figure as a pdf
#plt.savefig(output_dir + '/fastchem_fig.pdf')